import numpy as np, cupy as cp
from numba import cuda
import math

@cuda.jit
def kernel(position, potential):
    i = cuda.threadIdx.x + cuda.blockIdx.x * cuda.blockDim.x
    N = len(position)
    if i >= N: return
    mass = 1 / N
    potential_i = 0
    for j in range(N):
        if i != j:
            dx = position[i,0] - position[j,0]
            dy = position[i,1] - position[j,1]
            dz = position[i,2] - position[j,2]
            r = math.sqrt(dx**2 + dy**2 + dz**2)
            potential_i += - mass / r
    potential[i] = potential_i

def calculate_potential(position : np.ndarray) -> np.ndarray:
    threads_per_block = 32
    N = len(position)
    blocks_per_grid = int(np.ceil(N/threads_per_block))
    position  = cp.array(position)
    potential = cp.empty(N)
    kernel[blocks_per_grid, threads_per_block](position, potential)
    return cp.asnumpy(potential)