import ctypes, os, numpy as np
base_dir_name = os.path.dirname(os.path.realpath(__file__))
libsapporo = ctypes.cdll.LoadLibrary(os.path.join(base_dir_name, 'sapporo2/lib/libsapporo.so'))
os.environ['SAPPORO2_KERNEL_FILE'] = os.path.join(base_dir_name, 'sapporo2/lib/CUDAKernels/kernels.ptx')

npipes = libsapporo.g6_npipes_()

double_star = ctypes.POINTER(ctypes.c_double)
int_star    = ctypes.POINTER(ctypes.c_int)

class Sapporo:
    def __init__(self, dev_id=0):
        self.dev_id = ctypes.c_int(dev_id)
        libsapporo.g6_open_(ctypes.byref(self.dev_id))
        time = ctypes.c_double(0)
        libsapporo.g6_set_ti_(ctypes.byref(self.dev_id), ctypes.byref(time))
        self.j_particles_available = False
    def set_j_particles(self, position):
        self.j_particles_available = True
        N = len(position)
        self.N = N
        zeros = np.zeros((N, 3))
        self.zero_buffer = zeros.ctypes.data_as(double_star)
        mass = ctypes.c_double(1/N)

        self.index = np.arange(N, dtype=np.int32)
        self.h2    = np.empty(N)
        self.acc   = np.empty((N, 3))
        self.jrk   = np.empty((N, 3))
        self.pot   = np.empty(N)

        for j in range(N):
            j_cint = ctypes.c_int(j)
            libsapporo.g6_set_j_particle_(
                ctypes.byref(self.dev_id),
                ctypes.byref(j_cint),
                ctypes.byref(j_cint),
                self.zero_buffer,
                self.zero_buffer,
                ctypes.byref(mass),
                self.zero_buffer,
                self.zero_buffer,
                self.zero_buffer,
                self.zero_buffer,
                position[j].ctypes.data_as(double_star) # change to make nice
            )
    def calculate_potential(self, position):
        if not self.j_particles_available: raise RuntimeError('Set j particles first')
        N = len(position) # May not be the same as self.N!
        N_cint = ctypes.c_int(N)

        for i in range(0, N, npipes):
            ni_cint = ctypes.c_int(min(npipes, N-i))
            libsapporo.g6calc_firsthalf_(
                ctypes.byref(self.dev_id),
                ctypes.byref(N_cint),
                ctypes.byref(ni_cint),
                self.index[i:].ctypes.data_as(int_star),
                position[i:].ctypes.data_as(double_star),
                self.zero_buffer,
                self.zero_buffer,
                self.zero_buffer,
                self.zero_buffer, 
                self.zero_buffer,
                self.zero_buffer
            )
            libsapporo.g6calc_lasthalf_(
                ctypes.byref(self.dev_id),
                ctypes.byref(N_cint),
                ctypes.byref(ni_cint),
                self.index[i:].ctypes.data_as(int_star),
                position[i:].ctypes.data_as(double_star),
                self.zero_buffer,
                self.zero_buffer,
                self.h2[i:].ctypes.data_as(double_star),
                self.acc[i:].ctypes.data_as(double_star), 
                self.jrk[i:].ctypes.data_as(double_star),
                self.pot[i:].ctypes.data_as(double_star)
            )
        return self.pot

def calculate_potential(position):
    return RuntimeError('Please use the Sapporo class instead')