#include "hip/hip_runtime.h"
#include <algorithm>
#include <iostream>

__global__ void add(float *a, float *b)
{
  int i = blockDim.x * blockIdx.x + threadIdx.x;
  b[i] = a[i] + b[i];
}

int main()
{
  constexpr size_t n{1'000'000};
  float *a{new float[n]}, *b{new float[n]};
  std::fill(a, a+n, 2.);
  std::fill(b, b+n, 3.);

  float *a_dev, *b_dev;
  hipMalloc((void**)&a_dev, n*sizeof(float));
  hipMalloc((void**)&b_dev, n*sizeof(float));
  hipMemcpy(a_dev, a, n*sizeof(float), hipMemcpyHostToDevice);
  hipMemcpy(b_dev, b, n*sizeof(float), hipMemcpyHostToDevice);

  hipLaunchKernelGGL(add, dim3(n/64), dim3(64), 0, 0, a_dev, b_dev);

  hipMemcpy(b, b_dev, n*sizeof(float), hipMemcpyDeviceToHost);

  if (std::all_of(b, b+n, [](const auto x){ return x == 5.; }))
    std::cout << "Success!\n";
  else std::cout << "Failure :(\n";

  delete[] a;
  delete[] b;
  hipFree(a_dev);
  hipFree(b_dev);
}
