#include <algorithm>
#include <CL/sycl.hpp>
#include <iostream>
using namespace cl;

int main()
{
  constexpr size_t n{1'000'000};
  float *a{new float[n]}, *b{new float[n]};
  std::fill(a, a+n, 2.);
  std::fill(b, b+n, 3.);

  sycl::device device{sycl::default_selector()};
  sycl::context context{device};
  sycl::queue queue{context, device};

  float* a_dev{sycl::malloc_device<float>(n, device, context)};
  float* b_dev{sycl::malloc_device<float>(n, device, context)};

  queue.copy(a, a_dev, n);
  queue.copy(b, b_dev, n);
  queue.wait();

  queue.parallel_for(sycl::range<1>{n},
    [=](const auto i) { b_dev[i] = a_dev[i] + b_dev[i]; }
  );
  queue.wait();

  queue.copy(b_dev, b, n);
  queue.wait();

  if (std::all_of(b, b+n, [](const auto x){ return x == 5.; }))
    std::cout << "Success!\n";
  else std::cout << "Failure :(\n";
	
  delete[] a;
  delete[] b;
  sycl::free(a_dev, context);
  sycl::free(b_dev, context);
}