from numba import cuda
import numpy as np

@cuda.jit
def add(a, b):
  i = cuda.blockDim.x * cuda.blockIdx.x + cuda.threadIdx.x
  b[i] = a[i] + b[i]

if __name__ == '__main__':
  n = 1_000_000
  a = np.full(n, 2., dtype=np.float32)
  b = np.full(n, 3., dtype=np.float32)

  a_dev = cuda.to_device(a)
  b_dev = cuda.to_device(b)

  add[n//64,64](a_dev, b_dev)

  b = b_dev.copy_to_host()

  if all(b == 5.): print('Success!')
  else: print('Failure :(')
